<?php

namespace Drupal\mailgroup\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the Mail group entity.
 *
 * @ingroup mailgroup
 *
 * @ContentEntityType(
 *   id = "mailgroup",
 *   label = @Translation("Mail group"),
 *   label_collection = @Translation("Mail groups"),
 *   label_singular = @Translation("group"),
 *   label_plural = @Translation("groups"),
 *   label_count = @PluralTranslation(
 *     singular = "@count group",
 *     plural = "@count groups"
 *   ),
 *   bundle_label = @Translation("mail group type"),
 *   handlers = {
 *     "storage" = "Drupal\mailgroup\Entity\Storage\MailGroupStorage",
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\mailgroup\Entity\ListBuilder\MailGroupListBuilder",
 *     "views_data" = "Drupal\mailgroup\Entity\Views\MailGroupViewsData",
 *     "form" = {
 *       "default" = "Drupal\mailgroup\Entity\Form\MailGroupForm",
 *       "add" = "Drupal\mailgroup\Entity\Form\MailGroupForm",
 *       "edit" = "Drupal\mailgroup\Entity\Form\MailGroupForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "access" = "Drupal\mailgroup\Entity\Access\MailGroupAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\mailgroup\Entity\Routing\MailGroupRouteProvider",
 *     },
 *   },
 *   base_table = "mailgroup",
 *   admin_permission = "administer mail groups",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "type",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "owner" = "uid",
 *     "langcode" = "langcode",
 *     "status" = "status",
 *   },
 *   links = {
 *     "canonical" = "/mailgroup/{mailgroup}",
 *     "add-page" = "/admin/mailgroup/add",
 *     "add-form" = "/admin/mailgroup/add/{mailgroup_type}",
 *     "edit-form" = "/mailgroup/{mailgroup}/edit",
 *     "delete-form" = "/mailgroup/{mailgroup}/delete",
 *     "collection" = "/admin/mailgroup",
 *   },
 *   bundle_entity_type = "mailgroup_type",
 *   field_ui_base_route = "entity.mailgroup_type.edit_form"
 * )
 */
class MailGroup extends ContentEntityBase implements MailGroupInterface {
  use EntityOwnerTrait;
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getEmail() {
    return $this->get('email')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setEmail($email) {
    $this->set('email', $email);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function prependName() {
    return (bool) $this->get('prepend_name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setPrependName($prepend) {
    $this->set('prepend_name', $prepend);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getReplyTo() {
    return $this->get('reply_to')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setReplyTo($email) {
    $this->set('reply_to', $email);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getSendType() {
    return $this->get('send_type')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setSendType($send_type) {
    $this->set('send_type', $send_type);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getConnectionPlugin() {
    $plugin_id = $this->get('connection')->value;
    if ($plugin_id) {
      $connection_manager = \Drupal::service('plugin.manager.mailgroup_connection');
      $plugin = $connection_manager->createInstance($plugin_id, $this->getConnectionConfig());
    }
    else {
      $plugin = NULL;
    }

    return $plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function setConnectionPlugin($plugin) {
    $this->set('plugin', $plugin);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getConnectionConfig() {
    $config = $this->get('connection_config')->first();
    return $config ? $config->getValue() : [];
  }

  /**
   * {@inheritdoc}
   */
  public function setConnectionConfig(array $config) {
    return $this->set('connection_config', $config);
  }

  /**
   * {@inheritdoc}
   */
  public function isActive() {
    return (bool) $this->get('status')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setActive() {
    $this->set('status', TRUE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setInactive() {
    $this->set('status', FALSE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function testConnection() {
    /** @var \Drupal\mailgroup\ConnectionInterface $plugin */
    $plugin = $this->getConnectionPlugin();
    $status = $plugin->testConnection();

    return $status;
  }

  /**
   * {@inheritdoc}
   */
  public function getMessages() {
    $storage = $this->entityTypeManager()->getStorage('mailgroup_message');

    /** @var \Drupal\mailgroup\Entity\MailGroupMessageInterface[] $messages */
    $messages = $storage->loadByProperties([
      'gid' => $this->id(),
    ]);

    return $messages;
  }

  /**
   * {@inheritdoc}
   */
  public function getMembers() {
    $storage = $this->entityTypeManager()->getStorage('mailgroup_membership');

    /** @var \Drupal\mailgroup\Entity\MailGroupMembershipInterface[] $members */
    $members = $storage->loadByProperties([
      'gid' => $this->id(),
      'status' => TRUE,
    ]);

    return $members;
  }

  /**
   * {@inheritdoc}
   */
  public function getMemberEmails() {
    $emails = [];
    $members = $this->getMembers();

    foreach ($members as $member) {
      $emails[] = $member->getOwner()->getEmail();
    }

    return $emails;
  }

  /**
   * {@inheritdoc}
   */
  public function isMember($email) {
    /** @var \Drupal\mailgroup\Entity\Storage\MailGroupMembershipStorageInterface $storage */
    $storage = $this->entityTypeManager()->getStorage('mailgroup_membership');
    $memberships = $storage->loadByEmail($email, $this);

    return !empty($memberships) ? TRUE : FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    /** @var \Drupal\Core\Field\BaseFieldDefinition[] $fields */
    $fields = parent::baseFieldDefinitions($entity_type);
    $fields += static::ownerBaseFieldDefinitions($entity_type);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The name of the group.'))
      ->setRequired(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -10,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -10,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['uid']
      ->setLabel(t('Created by'))
      ->setDescription(t('The username of the group creator.'))
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 52,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['email'] = BaseFieldDefinition::create('email')
      ->setLabel(t('Email'))
      ->setDescription(t('The email address of the group.'))
      ->setRequired(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['prepend_name'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Prepend list name to the subject?'))
      ->setDefaultValue(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['reply_to'] = BaseFieldDefinition::create('list_integer')
      ->setLabel(t('Reply to'))
      ->setDescription(t('Who should replies to group messages be sent to?'))
      ->setRequired(TRUE)
      ->setSettings([
        'allowed_values' => [
          MailGroupInterface::REPLY_ALL => t('All members'),
          MailGroupInterface::REPLY_SENDER => t('Sender only'),
        ],
      ])
      ->setPropertyConstraints('value', [
        'AllowedValues' => [
          MailGroupInterface::REPLY_ALL,
          MailGroupInterface::REPLY_SENDER,
        ],
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['send_type'] = BaseFieldDefinition::create('list_integer')
      ->setLabel(t('Send type'))
      ->setDescription(t('Should messages be sent to members individually, or
        as a single message with all members added to the BCC?'))
      ->setRequired(TRUE)
      ->setSettings([
        'allowed_values' => [
          MailGroupInterface::SEND_INDIVIDUAL => t('Individual'),
          MailGroupInterface::SEND_GROUPED => t('Grouped'),
        ],
      ])
      ->setPropertyConstraints('value', [
        'AllowedValues' => [
          MailGroupInterface::SEND_INDIVIDUAL,
          MailGroupInterface::SEND_GROUPED,
        ],
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $connection_manager = \Drupal::service('plugin.manager.mailgroup_connection');
    $plugins = $connection_manager->getDefinitions();
    $plugin_options = [];
    foreach ($plugins as $plugin) {
      $plugin_options[$plugin['id']] = $plugin['label'];
    }

    $fields['connection'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Connection plugin'))
      ->setDescription(t('The connection plugin to use to receive mail.'))
      ->setSettings([
        'allowed_values' => $plugin_options,
      ])
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => 51,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['connection_config'] = BaseFieldDefinition::create('map')
      ->setLabel(t('Connection configuration'))
      ->setDescription(t('The configuration for the connection plugin.'));

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Active'))
      ->setDefaultValue(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => TRUE,
        ],
        'weight' => 20,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

}
